@@ -1,3 +1,11 @@
+# -*- coding: utf-8 -*-
+# Copyright (c) 2006-2016 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2012, 2014 Google, Inc.
+# Copyright (c) 2013-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
+# Copyright (c) 2016 Moises Lopez - https://www.vauxoo.com/ <moylop260@vauxoo.com>
+# Copyright (c) 2016 Łukasz Rogalski <rogalski.91@gmail.com>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -5,14 +13,15 @@
 """
 from __future__ import generators
 
+import collections
 import sys
-from collections import defaultdict
 
 import six
 
 import astroid
 from astroid.bases import Generator, BUILTINS
 from astroid.exceptions import InconsistentMroError, DuplicateBasesError
+from astroid import decorators
 from astroid import objects
 from astroid.scoped_nodes import function_to_method
 from pylint.interfaces import IAstroidChecker
@@ -23,23 +32,170 @@ from pylint.checkers.utils import (
     is_attr_protected, node_frame_class, is_builtin_object,
     decorated_with_property, unimplemented_abstract_methods,
     decorated_with, class_is_abstract,
-    safe_infer, has_known_bases)
-from pylint.utils import deprecated_option, get_global_option
+    safe_infer, has_known_bases, is_iterable, is_comprehension)
+from pylint.utils import get_global_option
 
 
 if sys.version_info >= (3, 0):
     NEXT_METHOD = '__next__'
 else:
     NEXT_METHOD = 'next'
-ITER_METHODS = ('__iter__', '__getitem__')
 INVALID_BASE_CLASSES = {'bool', 'range', 'slice', 'memoryview'}
 
 
-def _get_method_args(method):
-    args = method.args.args
+# Dealing with useless override detection, with regard
+# to parameters vs arguments
+
+_CallSignature = collections.namedtuple(
+    '_CallSignature', 'args kws starred_args starred_kws')
+_ParameterSignature = collections.namedtuple(
+    '_ParameterSignature',
+    'args kwonlyargs varargs kwargs',
+)
+
+
+def _signature_from_call(call):
+    kws = {}
+    args = []
+    starred_kws = []
+    starred_args = []
+    for keyword in call.keywords or []:
+        arg, value = keyword.arg, keyword.value
+        if arg is None and isinstance(value, astroid.Name):
+            # Starred node and we are interested only in names,
+            # otherwise some transformation might occur for the parameter.
+            starred_kws.append(value.name)
+        elif isinstance(value, astroid.Name):
+            kws[arg] = value.name
+        else:
+            kws[arg] = None
+
+    for arg in call.args:
+        if isinstance(arg, astroid.Starred) and isinstance(arg.value, astroid.Name):
+            # Positional variadic and a name, otherwise some transformation
+            # might have occurred.
+            starred_args.append(arg.value.name)
+        elif isinstance(arg, astroid.Name):
+            args.append(arg.name)
+        else:
+            args.append(None)
+
+    return _CallSignature(args, kws, starred_args, starred_kws)
+
+
+def _signature_from_arguments(arguments):
+    kwarg = arguments.kwarg
+    vararg = arguments.vararg
+    args = [arg.name for arg in arguments.args if arg.name != 'self']
+    kwonlyargs = [arg.name for arg in arguments.kwonlyargs]
+    return _ParameterSignature(args, kwonlyargs, vararg, kwarg)
+
+
+def _definition_equivalent_to_call(definition, call):
+    '''Check if a definition signature is equivalent to a call.'''
+    if definition.kwargs:
+        same_kw_variadics = definition.kwargs in call.starred_kws
+    else:
+        same_kw_variadics = not call.starred_kws
+    if definition.varargs:
+        same_args_variadics = definition.varargs in call.starred_args
+    else:
+        same_args_variadics = not call.starred_args
+    same_kwonlyargs = all(kw in call.kws for kw in definition.kwonlyargs)
+    same_args = definition.args == call.args
+
+    no_additional_kwarg_arguments = True
+    if call.kws:
+        for keyword in call.kws:
+            is_arg = keyword in call.args
+            is_kwonly = keyword in definition.kwonlyargs
+            if not is_arg and not is_kwonly:
+                # Maybe this argument goes into **kwargs,
+                # or it is an extraneous argument.
+                # In any case, the signature is different than
+                # the call site, which stops our search.
+                no_additional_kwarg_arguments = False
+                break
+
+    return all((
+        same_args,
+        same_kwonlyargs,
+        same_args_variadics,
+        same_kw_variadics,
+        no_additional_kwarg_arguments,
+    ))
+
+# Deal with parameters overridding in two methods.
+
+def _positional_parameters(method):
+    positional = method.args.args
     if method.type in ('classmethod', 'method'):
-        return len(args) - 1
-    return len(args)
+        positional = positional[1:]
+    return positional
+
+
+def _has_different_parameters(original, overridden, dummy_parameter_regex):
+    zipped = six.moves.zip_longest(original, overridden)
+    for original_param, overridden_param in zipped:
+        params = (original_param, overridden_param)
+        if not all(params):
+            return True
+
+        names = [param.name for param in params]
+        if any(map(dummy_parameter_regex.match, names)):
+            continue
+        if original_param.name != overridden_param.name:
+            return True
+    return False
+
+
+def _different_parameters(original, overridden, dummy_parameter_regex):
+    """Determine if the two methods have different parameters
+
+    They are considered to have different parameters if:
+
+       * they have different positional parameters, including different names
+
+       * one of the methods is having variadics, while the other is not
+
+       * they have different keyword only parameters.
+
+    """
+    original_parameters = _positional_parameters(original)
+    overridden_parameters = _positional_parameters(overridden)
+
+    different_positional = _has_different_parameters(
+        original_parameters,
+        overridden_parameters,
+        dummy_parameter_regex)
+    different_kwonly = _has_different_parameters(
+        original.args.kwonlyargs,
+        overridden.args.kwonlyargs,
+        dummy_parameter_regex)
+    if original.name in PYMETHODS:
+        # Ignore the difference for special methods. If the parameter
+        # numbers are different, then that is going to be caught by
+        # unexpected-special-method-signature.
+        # If the names are different, it doesn't matter, since they can't
+        # be used as keyword arguments anyway.
+        different_positional = different_kwonly = False
+
+    # Both or none should have extra variadics, otherwise the method
+    # loses or gains capabilities that are not reflected into the parent method,
+    # leading to potential inconsistencies in the code.
+    different_kwarg = sum(
+        1 for param in (original.args.kwarg, overridden.args.kwarg)
+        if not param) == 1
+    different_vararg = sum(
+        1 for param in (original.args.vararg, overridden.args.vararg)
+        if not param) == 1
+
+    return any((
+        different_positional,
+        different_kwarg,
+        different_vararg,
+        different_kwonly
+    ))
 
 
 def _is_invalid_base_class(cls):
@@ -120,6 +276,7 @@ def _is_attribute_property(name, klass):
             return True
     return False
 
+
 def _has_bare_super_call(fundef_node):
     for call in fundef_node.nodes_of_class(astroid.Call):
         func = call.func
@@ -129,6 +286,7 @@ def _has_bare_super_call(fundef_node):
             return True
     return False
 
+
 def _safe_infer_call_result(node, caller, context=None):
     """
     Safely infer the return value of a function.
@@ -151,6 +309,7 @@ def _safe_infer_call_result(node, caller, context=None):
     except StopIteration:
         return value
 
+
 MSGS = {
     'F0202': ('Unable to check methods signature (%s / %s)',
               'method-check-failed',
@@ -195,7 +354,7 @@ MSGS = {
               'from regular instance methods.'),
     'C0203': ('Metaclass method %s should have %s as first argument',
               'bad-mcs-method-argument',
-              'Used when a metaclass method has a first agument named '
+              'Used when a metaclass method has a first argument named '
               'differently than the value specified in valid-classmethod-first'
               '-arg option (default to "cls"), recommended to easily '
               'differentiate them from regular instance methods.'),
@@ -217,7 +376,7 @@ MSGS = {
               'Used when a method doesn\'t use its bound instance, and so could '
               'be written as a function.'
              ),
-    'W0221': ('Arguments number differs from %s %r method',
+    'W0221': ('Parameters differ from %s %r method',
               'arguments-differ',
               'Used when a method has a different number of arguments than in '
               'the implemented interface or in an overridden method.'),
@@ -242,6 +401,11 @@ MSGS = {
               'non-parent-init-called',
               'Used when an __init__ method is called on a class which is not '
               'in the direct ancestors for the analysed class.'),
+    'W0235': ('Useless super delegation in method %r',
+              'useless-super-delegation',
+              'Used whenever we can detect that an overridden method is useless, '
+              'relying on super() delegation to do the same thing as another method '
+              'from the MRO.'),
     'E0236': ('Invalid object %r in __slots__, must contain '
               'only non empty strings',
               'invalid-slots-object',
@@ -260,7 +424,7 @@ MSGS = {
               'class.'),
     'E0240': ('Inconsistent method resolution order for class %r',
               'inconsistent-mro',
-              'Used when a class has an inconsistent method resolutin order.'),
+              'Used when a class has an inconsistent method resolution order.'),
     'E0241': ('Duplicate bases for class %r',
               'duplicate-bases',
               'Used when a class has duplicate bases.'),
@@ -272,9 +436,35 @@ MSGS = {
               'no-staticmethod-decorator',
               'Used when a static method is defined without using the decorator '
               'syntax.'),
+    'C0205': ('Class __slots__ should be a non-string iterable',
+              'single-string-used-for-slots',
+              'Used when a class __slots__ is a simple string, rather '
+              'than an iterable.'),
     }
 
 
+class ScopeAccessMap(object):
+    """Store the accessed variables per scope."""
+
+    def __init__(self):
+        self._scopes = collections.defaultdict(
+            lambda: collections.defaultdict(list)
+        )
+
+    def set_accessed(self, node):
+        """Set the given node as accessed."""
+
+        frame = node_frame_class(node)
+        if frame is None:
+            # The node does not live in a class.
+            return
+        self._scopes[frame][node.attrname].append(node)
+
+    def accessed(self, scope):
+        """Get the accessed variables for the given scope."""
+        return self._scopes.get(scope, {})
+
+
 class ClassChecker(BaseChecker):
     """checks for :
     * methods without self as first argument
@@ -292,13 +482,7 @@ class ClassChecker(BaseChecker):
     msgs = MSGS
     priority = -2
     # configuration options
-    options = (('ignore-iface-methods',
-                deprecated_option(opt_type="csv",
-                                  help_msg="This is deprecated, because "
-                                           "it is not used anymore.",
-                                  deprecation_msg="This option %r will be "
-                                                  "removed in Pylint 2.0")),
-               ('defining-attr-methods',
+    options = (('defining-attr-methods',
                 {'default' : ('__init__', '__new__', 'setUp'),
                  'type' : 'csv',
                  'metavar' : '<method names>',
@@ -332,14 +516,23 @@ a metaclass class method.'}
 
     def __init__(self, linter=None):
         BaseChecker.__init__(self, linter)
-        self._accessed = []
+        self._accessed = ScopeAccessMap()
         self._first_attrs = []
         self._meth_could_be_func = None
 
+    @decorators.cachedproperty
+    def _dummy_rgx(self):
+        return get_global_option(
+            self, 'dummy-variables-rgx', default=None)
+
+    @decorators.cachedproperty
+    def _ignore_mixin(self):
+        return get_global_option(
+            self, 'ignore-mixin-members', default=True)
+
     def visit_classdef(self, node):
         """init visit variable _accessed
         """
-        self._accessed.append(defaultdict(list))
         self._check_bases_classes(node)
         # if not an exception or a metaclass
         if node.type == 'class' and has_known_bases(node):
@@ -387,15 +580,13 @@ a metaclass class method.'}
         access to existent members
         """
         # check access to existent members on non metaclass classes
-        ignore_mixins = get_global_option(self, 'ignore-mixin-members',
-                                          default=True)
-        if ignore_mixins and cnode.name[-5:].lower() == 'mixin':
+        if self._ignore_mixin and cnode.name[-5:].lower() == 'mixin':
             # We are in a mixin class. No need to try to figure out if
             # something is missing, since it is most likely that it will
             # miss.
             return
 
-        accessed = self._accessed.pop()
+        accessed = self._accessed.accessed(cnode)
         if cnode.type != 'metaclass':
             self._check_accessed_members(cnode, accessed)
         # checks attributes are defined in an allowed method such as __init__
@@ -447,6 +638,9 @@ a metaclass class method.'}
         # ignore actual functions
         if not node.is_method():
             return
+
+        self._check_useless_super_delegation(node)
+
         klass = node.parent.frame()
         self._meth_could_be_func = True
         # check first argument is self if this is actually a method
@@ -494,23 +688,87 @@ a metaclass class method.'}
 
     visit_asyncfunctiondef = visit_functiondef
 
+    def _check_useless_super_delegation(self, function):
+        '''Check if the given function node is an useless method override
+
+        We consider it *useless* if it uses the super() builtin, but having
+        nothing additional whatsoever than not implementing the method at all.
+        If the method uses super() to delegate an operation to the rest of the MRO,
+        and if the method called is the same as the current one, the arguments
+        passed to super() are the same as the parameters that were passed to
+        this method, then the method could be removed altogether, by letting
+        other implementation to take precedence.
+        '''
+
+        if not function.is_method():
+            return
+
+        if function.decorators:
+            # With decorators is a change of use
+            return
+
+        body = function.body
+        if len(body) != 1:
+            # Multiple statements, which means this overridden method
+            # could do multiple things we are not aware of.
+            return
+
+        statement = body[0]
+        if not isinstance(statement, (astroid.Expr, astroid.Return)):
+            # Doing something else than what we are interested into.
+            return
+
+        call = statement.value
+        if not isinstance(call, astroid.Call):
+            return
+        if not isinstance(call.func, astroid.Attribute):
+            # Not a super() attribute access.
+            return
+
+        # Should be a super call.
+        try:
+            super_call = next(call.func.expr.infer())
+        except astroid.InferenceError:
+            return
+        else:
+            if not isinstance(super_call, objects.Super):
+                return
+
+        # The name should be the same.
+        if call.func.attrname != function.name:
+            return
+
+        # Should be a super call with the MRO pointer being the current class
+        # and the type being the current instance.
+        current_scope = function.parent.scope()
+        if super_call.mro_pointer != current_scope:
+            return
+        if not isinstance(super_call.type, astroid.Instance):
+            return
+        if super_call.type.name != current_scope.name:
+            return
+
+        # Detect if the parameters are the same as the call's arguments.
+        params = _signature_from_arguments(function.args)
+        args = _signature_from_call(call)
+        if _definition_equivalent_to_call(params, args):
+            self.add_message('useless-super-delegation', node=function,
+                             args=(function.name, ))
+
     def _check_slots(self, node):
         if '__slots__' not in node.locals:
             return
         for slots in node.igetattr('__slots__'):
             # check if __slots__ is a valid type
-            for meth in ITER_METHODS:
-                try:
-                    slots.getattr(meth)
-                    break
-                except astroid.NotFoundError:
-                    continue
-            else:
+            if slots is astroid.YES:
+                continue
+            if not is_iterable(slots) and not is_comprehension(slots):
                 self.add_message('invalid-slots', node=node)
                 continue
 
             if isinstance(slots, astroid.Const):
                 # a string, ignore the following checks
+                self.add_message('single-string-used-for-slots', node=node)
                 continue
             if not hasattr(slots, 'itered'):
                 # we can't obtain the values, maybe a .deque?
@@ -570,10 +828,9 @@ a metaclass class method.'}
         class member from outside its class (but ignore __special__
         methods)
         """
-        attrname = node.attrname
         # Check self
         if self.is_first_attr(node):
-            self._accessed[-1][attrname].append(node)
+            self._accessed.set_accessed(node)
             return
         if not self.linter.is_message_enabled('protected-access'):
             return
@@ -582,7 +839,7 @@ a metaclass class method.'}
 
     def visit_assignattr(self, node):
         if isinstance(node.assign_type(), astroid.AugAssign) and self.is_first_attr(node):
-            self._accessed[-1][node.attrname].append(node)
+            self._accessed.set_accessed(node)
         self._check_in_slots(node)
 
     def _check_in_slots(self, node):
@@ -930,16 +1187,14 @@ a metaclass class method.'}
                              args=(method1, refmethod), node=method1)
             return
 
-        instance = cls.instanciate_class()
+        instance = cls.instantiate_class()
         method1 = function_to_method(method1, instance)
         refmethod = function_to_method(refmethod, instance)
 
         # Don't care about functions with unknown argument (builtins).
         if method1.args.args is None or refmethod.args.args is None:
             return
-        # If we use *args, **kwargs, skip the below checks.
-        if method1.args.vararg or method1.args.kwarg:
-            return
+
         # Ignore private to class methods.
         if is_attr_private(method1.name):
             return
@@ -951,9 +1206,9 @@ a metaclass class method.'}
                         decorator.attrname == 'setter'):
                     return
 
-        method1_args = _get_method_args(method1)
-        refmethod_args = _get_method_args(refmethod)
-        if method1_args != refmethod_args:
+        if _different_parameters(
+                refmethod, method1,
+                dummy_parameter_regex=self._dummy_rgx):
             self.add_message('arguments-differ',
                              args=(class_type, method1.name),
                              node=method1)
@@ -1016,8 +1271,8 @@ class SpecialMethodsChecker(BaseChecker):
         if expected_params is None:
             # This can support a variable number of parameters.
             return
-        if not len(node.args.args) and not node.args.vararg:
-            # Method has no parameter, will be catched
+        if not node.args.args and not node.args.vararg:
+            # Method has no parameter, will be caught
             # by no-method-argument.
             return
 
@@ -1088,7 +1343,7 @@ class SpecialMethodsChecker(BaseChecker):
 
     def _check_len(self, node):
         inferred = _safe_infer_call_result(node, node)
-        if inferred is None or inferred is astroid.YES:
+        if not inferred:
             return
 
         if not isinstance(inferred, astroid.Const):
