@@ -1,14 +1,22 @@
-# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2012-2014 Google, Inc.
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
+# Copyright (c) 2015 Noam Yorav-Raphael <noamraph@gmail.com>
+# Copyright (c) 2015 Cezar <celnazli@bitdefender.com>
+# Copyright (c) 2015 James Morgensen <james.morgensen@gmail.com>
+# Copyright (c) 2016 Moises Lopez - https://www.vauxoo.com/ <moylop260@vauxoo.com>
+# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """imports checkers for Python code"""
 
 import collections
+from distutils import sysconfig
 import os
 import sys
-from distutils import sysconfig
 
 import six
 
@@ -18,7 +26,8 @@ from astroid.modutils import (get_module_part, is_standard_module)
 import isort
 
 from pylint.interfaces import IAstroidChecker
-from pylint.utils import EmptyReport, get_global_option
+from pylint.utils import get_global_option
+from pylint.exceptions import EmptyReportError
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import (
     check_messages,
@@ -75,23 +84,27 @@ def _get_first_import(node, context, name, base, level, alias):
                 found = True
                 break
         elif isinstance(first, astroid.ImportFrom):
-            if level != first.level:
-                continue
-
-            for imported_name, imported_alias in first.names:
-                if fullname == '%s.%s' % (first.modname, imported_name):
-                    found = True
+            if level == first.level:
+                for imported_name, imported_alias in first.names:
+                    if fullname == '%s.%s' % (first.modname, imported_name):
+                        found = True
+                        break
+                    if name != '*' and name == imported_name and not (alias or imported_alias):
+                        found = True
+                        break
+                if found:
                     break
-                if name != '*' and name == imported_name:
-                    if alias or imported_alias:
-                        continue
-                    found = True
-                    break
-            if found:
-                break
     if found and not are_exclusive(first, node):
         return first
 
+
+def _ignore_import_failure(node, modname, ignored_modules):
+    for submodule in _qualified_names(modname):
+        if submodule in ignored_modules:
+            return True
+
+    return node_ignores_exception(node, ImportError)
+
 # utilities to represents import dependencies as tree and dot graph ###########
 
 def _make_tree_defs(mod_files_list):
@@ -115,7 +128,7 @@ def _repr_tree_defs(data, indent_str=None):
         if not files:
             files = ''
         else:
-            files = '(%s)' % ','.join(files)
+            files = '(%s)' % ','.join(sorted(files))
         if indent_str is None:
             lines.append('%s %s' % (mod, files))
             sub_indent_str = '  '
@@ -139,10 +152,10 @@ def _dependencies_graph(filename, dep_info):
     for modname, dependencies in sorted(six.iteritems(dep_info)):
         done[modname] = 1
         printer.emit_node(modname)
-        for modname in dependencies:
-            if modname not in done:
-                done[modname] = 1
-                printer.emit_node(modname)
+        for depmodname in dependencies:
+            if depmodname not in done:
+                done[depmodname] = 1
+                printer.emit_node(depmodname)
     for depmodname, dependencies in sorted(six.iteritems(dep_info)):
         for modname in dependencies:
             printer.emit_edge(modname, depmodname)
@@ -165,6 +178,10 @@ MSGS = {
               'import-error',
               'Used when pylint has been unable to import a module.',
               {'old_names': [('F0401', 'import-error')]}),
+    'E0402': ('Attempted relative import beyond top-level package',
+              'relative-beyond-top-level',
+              'Used when a relative import tries to access too many levels '
+              'in the current package.'),
     'R0401': ('Cyclic import (%s)',
               'cyclic-import',
               'Used when a cyclic import between two or more modules is \
@@ -231,8 +248,10 @@ class ImportsChecker(BaseChecker):
 
     if six.PY2:
         deprecated_modules = ('regsub', 'TERMIOS', 'Bastion', 'rexec')
-    else:
+    elif sys.version_info < (3, 5):
         deprecated_modules = ('optparse', )
+    else:
+        deprecated_modules = ('optparse', 'tkinter.tix')
     options = (('deprecated-modules',
                 {'default' : deprecated_modules,
                  'type' : 'csv',
@@ -519,7 +538,7 @@ given file (report RP0402 must not be disabled)'}
         return any(astroid.are_exclusive(import_node, node)
                    for import_node in imports)
 
-    def _check_imports_order(self, node):
+    def _check_imports_order(self, _module_node):
         """Checks imports of module `node` are grouped by category
 
         Imports must follow this order: standard, 3rd party, local
@@ -567,22 +586,21 @@ given file (report RP0402 must not be disabled)'}
     def _get_imported_module(self, importnode, modname):
         try:
             return importnode.do_import_module(modname)
-        except astroid.InferenceError as ex:
-            dotted_modname = _get_import_name(importnode, modname)
-            if str(ex) != modname:
-                args = '%r (%s)' % (dotted_modname, ex)
-            else:
-                args = repr(dotted_modname)
+        except astroid.TooManyLevelsError:
+            if _ignore_import_failure(importnode, modname, self._ignored_modules):
+                return None
 
-            for submodule in _qualified_names(modname):
-                if submodule in self._ignored_modules:
-                    return None
+            self.add_message('relative-beyond-top-level', node=importnode)
 
+        except astroid.AstroidBuildingException:
+            if _ignore_import_failure(importnode, modname, self._ignored_modules):
+                return None
             if not self.config.analyse_fallback_blocks and is_from_fallback_block(importnode):
                 return None
 
-            if not node_ignores_exception(importnode, ImportError):
-                self.add_message("import-error", args=args, node=importnode)
+            dotted_modname = _get_import_name(importnode, modname)
+            self.add_message('import-error', args=repr(dotted_modname),
+                             node=importnode)
 
     def _check_relative_import(self, modnode, importnode, importedmodnode,
                                importedasname):
@@ -654,28 +672,31 @@ given file (report RP0402 must not be disabled)'}
         if root is not frame:
             contexts.append((root, None))
 
-        for context, level in contexts:
+        for known_context, known_level in contexts:
             for name, alias in node.names:
-                first = _get_first_import(node, context, name, basename, level, alias)
+                first = _get_first_import(
+                    node, known_context,
+                    name, basename,
+                    known_level, alias)
                 if first is not None:
                     self.add_message('reimported', node=node,
                                      args=(name, first.fromlineno))
 
-    def _report_external_dependencies(self, sect, _, dummy):
+    def _report_external_dependencies(self, sect, _, _dummy):
         """return a verbatim layout for displaying dependencies"""
         dep_info = _make_tree_defs(six.iteritems(self._external_dependencies_info()))
         if not dep_info:
-            raise EmptyReport()
+            raise EmptyReportError()
         tree_str = _repr_tree_defs(dep_info)
         sect.append(VerbatimText(tree_str))
 
-    def _report_dependencies_graph(self, sect, _, dummy):
+    def _report_dependencies_graph(self, sect, _, _dummy):
         """write dependencies as a dot (graphviz) file"""
         dep_info = self.stats['dependencies']
         if not dep_info or not (self.config.import_graph
                                 or self.config.ext_import_graph
                                 or self.config.int_import_graph):
-            raise EmptyReport()
+            raise EmptyReportError()
         filename = self.config.import_graph
         if filename:
             _make_graph(filename, dep_info, sect, '')
