@@ -1,5 +1,7 @@
-# Copyright (c) 2014-2016 Michal Nowikowski
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015 Pavel Roskin <proski@gnu.org>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -7,10 +9,10 @@
 """
 
 import os
-import re
+import sys
 import tokenize
 import string
-import sys
+import re
 
 try:
     import enchant
@@ -127,6 +129,15 @@ class SpellingChecker(BaseTokenChecker):
 
     def _check_spelling(self, msgid, line, line_num):
         line2 = line.strip()
+        if line_num == 1 and line.startswith('#!/'):
+            # Skip shebang lines
+            return
+        if line_num <= 2 and re.match("^[ \t\v]*#.*?coding[:=][ \t]*([-_.a-zA-Z0-9]+)", line):
+            # Skip coding definition lines
+            return
+        if line.startswith('# pylint:'):
+            # Skip pylint enable/disable comments
+            return
         # Replace ['afadf with afadf (but preserve don't)
         line2 = re.sub("'([^a-zA-Z]|$)", " ", line2)
         # Replace afadf'] with afadf (but preserve don't)
@@ -137,13 +148,13 @@ class SpellingChecker(BaseTokenChecker):
         words = []
         for word in line2.split():
             # Skip words with digits.
-            if len(re.findall(r"\d", word)) > 0:
+            if re.findall(r"\d", word):
                 continue
 
             # Skip words with mixed big and small letters,
             # they are probaly class names.
-            if (len(re.findall("[A-Z]", word)) > 0 and
-                    len(re.findall("[a-z]", word)) > 0 and
+            if (re.findall("[A-Z]", word) and
+                    re.findall("[a-z]", word) and
                     len(word) > 2):
                 continue
 
