@@ -1,5 +1,15 @@
-# Copyright (c) 2006-2016 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# -*- coding: utf-8 -*-
+# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2013-2014, 2016 Google, Inc.
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014 Holger Peters <email@holger-peters.de>
+# Copyright (c) 2014 David Shea <dshea@redhat.com>
+# Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
+# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
+# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
+# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2016 Jürgen Hermann <jh@web.de>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -8,16 +18,21 @@
 
 import collections
 import fnmatch
+import heapq
+import itertools
+import operator
 import re
 import shlex
 import sys
 
+import editdistance
 import six
 
 import astroid
 import astroid.context
 import astroid.arguments
 from astroid import exceptions
+from astroid.interpreter import dunder_lookup
 from astroid import objects
 from astroid import bases
 
@@ -28,9 +43,14 @@ from pylint.checkers.utils import (
     decorated_with, node_ignores_exception,
     is_iterable, is_mapping, supports_membership_test,
     is_comprehension, is_inside_abstract_class,
-    supports_subscript,
+    supports_getitem,
+    supports_setitem,
+    supports_delitem,
     safe_infer,
-    has_known_bases)
+    has_known_bases,
+    is_builtin_object,
+    singledispatch)
+
 
 BUILTINS = six.moves.builtins.__name__
 STR_FORMAT = "%s.str.format" % BUILTINS
@@ -40,8 +60,8 @@ def _unflatten(iterable):
     for index, elem in enumerate(iterable):
         if (isinstance(elem, collections.Sequence) and
                 not isinstance(elem, six.string_types)):
-            for elem in _unflatten(elem):
-                yield elem
+            for single_elem in _unflatten(elem):
+                yield single_elem
         elif elem and not index:
             # We're interested only in the first element.
             yield elem
@@ -75,8 +95,71 @@ def _is_owner_ignored(owner, name, ignored_classes, ignored_modules):
     return any(name == ignore or qname == ignore for ignore in ignored_classes)
 
 
+@singledispatch
+def _node_names(node):
+    # TODO: maybe we need an ABC for checking if an object is a scoped node
+    # or not?
+    if not hasattr(node, 'locals'):
+        return []
+    return node.locals.keys()
+
+
+@_node_names.register(astroid.ClassDef)
+@_node_names.register(astroid.Instance)
+def _(node):
+    values = itertools.chain(node.instance_attrs.keys(), node.locals.keys())
+
+    try:
+        mro = node.mro()[1:]
+    except (NotImplementedError, TypeError):
+        mro = node.ancestors()
+
+    other_values = [value for cls in mro for value in _node_names(cls)]
+    return itertools.chain(values, other_values)
+
+
+def _similar_names(owner, attrname, distance_threshold, max_choices):
+    """Given an owner and a name, try to find similar names
+
+    The similar names are searched given a distance metric and only
+    a given number of choices will be returned.
+    """
+    possible_names = []
+    names = _node_names(owner)
+
+    for name in names:
+        if name == attrname:
+            continue
+
+        distance = editdistance.eval(attrname, name)
+        if distance <= distance_threshold:
+            possible_names.append((name, distance))
+
+    # Now get back the values with a minimum, up to the given
+    # limit or choices.
+    picked = [name for (name, _) in
+              heapq.nsmallest(max_choices, possible_names,
+                              key=operator.itemgetter(1))]
+    return sorted(picked)
+
+
+def _missing_member_hint(owner, attrname, distance_threshold, max_choices):
+    names = _similar_names(owner, attrname, distance_threshold, max_choices)
+    if not names:
+        # No similar name.
+        return ""
+
+    names = list(map(repr, names))
+    if len(names) == 1:
+        names = ", ".join(names)
+    else:
+        names = "one of {} or {}".format(", ".join(names[:-1]), names[-1])
+
+    return "; maybe {}?".format(names)
+
+
 MSGS = {
-    'E1101': ('%s %r has no %r member',
+    'E1101': ('%s %r has no %r member%s',
               'no-member',
               'Used when a variable is accessed for an unexistent member.',
               {'old_names': [('E1103', 'maybe-no-member')]}),
@@ -129,7 +212,7 @@ MSGS = {
               'the context manager protocol(__enter__/__exit__).'),
     'E1130': ('%s',
               'invalid-unary-operand-type',
-              'Emitted when an unary operand is used on an object which does not '
+              'Emitted when a unary operand is used on an object which does not '
               'support this type of operation'),
     'E1131': ('%s',
               'unsupported-binary-operation',
@@ -140,12 +223,25 @@ MSGS = {
               'Emitted when a function call got multiple values for a keyword.'),
     'E1135': ("Value '%s' doesn't support membership test",
               'unsupported-membership-test',
-              'Emitted when an instance in membership test expression doesn\'t'
+              'Emitted when an instance in membership test expression doesn\'t '
               'implement membership protocol (__contains__/__iter__/__getitem__)'),
     'E1136': ("Value '%s' is unsubscriptable",
               'unsubscriptable-object',
               "Emitted when a subscripted value doesn't support subscription"
               "(i.e. doesn't define __getitem__ method)"),
+    'E1137': ("%r does not support item assignment",
+              'unsupported-assignment-operation',
+              "Emitted when an object does not support item assignment "
+              "(i.e. doesn't define __setitem__ method)"),
+    'E1138': ("%r does not support item deletion",
+              'unsupported-delete-operation',
+              "Emitted when an object does not support item deletion "
+              "(i.e. doesn't define __delitem__ method)"),
+    'E1139': ('Invalid metaclass %r used',
+              'invalid-metaclass',
+              'Emitted whenever we can detect that a class is using, '
+              'as a metaclass, something which might be invalid for using as '
+              'a metaclass.'),
     }
 
 # builtin sequence types in Python 2 and 3.
@@ -237,6 +333,134 @@ def _determine_callable(callable_obj):
     else:
         raise ValueError
 
+
+def _has_parent_of_type(node, node_type, statement):
+    """Check if the given node has a parent of the given type."""
+    parent = node.parent
+    while not isinstance(parent, node_type) and statement.parent_of(parent):
+        parent = parent.parent
+    return isinstance(parent, node_type)
+
+
+def _is_name_used_as_variadic(name, variadics):
+    """Check if the given name is used as a variadic argument."""
+    return any(variadic.value == name or variadic.value.parent_of(name)
+               for variadic in variadics)
+
+
+def _no_context_variadic_keywords(node):
+    statement = node.statement()
+    scope = node.scope()
+    variadics = ()
+
+    if not isinstance(scope, astroid.FunctionDef):
+        return False
+
+    if isinstance(statement, astroid.Expr) and isinstance(statement.value, astroid.Call):
+        call = statement.value
+        variadics = call.keywords or ()
+
+    return _no_context_variadic(node, scope.args.kwarg, astroid.Keyword, variadics)
+
+
+def _no_context_variadic_positional(node):
+    statement = node.statement()
+    scope = node.scope()
+    variadics = ()
+
+    if not isinstance(scope, astroid.FunctionDef):
+        return False
+
+    if isinstance(statement, astroid.Expr) and isinstance(statement.value, astroid.Call):
+        call = statement.value
+        variadics = call.starargs
+
+    return _no_context_variadic(node, scope.args.vararg, astroid.Starred, variadics)
+
+
+def _no_context_variadic(node, variadic_name, variadic_type, variadics):
+    """Verify if the given call node has variadic nodes without context
+
+    This is a workaround for handling cases of nested call functions
+    which don't have the specific call context at hand.
+    Variadic arguments (variable positional arguments and variable
+    keyword arguments) are inferred, inherently wrong, by astroid
+    as a Tuple, respectively a Dict with empty elements.
+    This can lead pylint to believe that a function call receives
+    too few arguments.
+    """
+    statement = node.statement()
+    for name in statement.nodes_of_class(astroid.Name):
+        if name.name != variadic_name:
+            continue
+
+        inferred = safe_infer(name)
+        if isinstance(inferred, (astroid.List, astroid.Tuple)):
+            length = len(inferred.elts)
+        elif isinstance(inferred, astroid.Dict):
+            length = len(inferred.items)
+        else:
+            continue
+
+        inferred_statement = inferred.statement()
+        if not length and isinstance(inferred_statement, astroid.FunctionDef):
+            is_in_starred_context = _has_parent_of_type(node, variadic_type, statement)
+            used_as_starred_argument = _is_name_used_as_variadic(name, variadics)
+            if is_in_starred_context or used_as_starred_argument:
+                return True
+    return False
+
+
+def _is_invalid_metaclass(metaclass):
+    try:
+        mro = metaclass.mro()
+    except NotImplementedError:
+        # Cannot have a metaclass which is not a newstyle class.
+        return True
+    else:
+        if not any(is_builtin_object(cls) and cls.name == 'type'
+                   for cls in mro):
+            return True
+    return False
+
+
+def _infer_from_metaclass_constructor(cls, func):
+    """Try to infer what the given *func* constructor is building
+
+    :param astroid.FunctionDef func:
+        A metaclass constructor. Metaclass definitions can be
+        functions, which should accept three arguments, the name of
+        the class, the bases of the class and the attributes.
+        The function could return anything, but usually it should
+        be a proper metaclass.
+    :param astroid.ClassDef cls:
+        The class for which the *func* parameter should generate
+        a metaclass.
+    :returns:
+        The class generated by the function or None,
+        if we couldn't infer it.
+    :rtype: astroid.ClassDef
+    """
+    context = astroid.context.InferenceContext()
+
+    class_bases = astroid.List()
+    class_bases.postinit(elts=cls.bases)
+
+    attrs = astroid.Dict()
+    local_names = [(name, values[-1]) for name, values in cls.locals.items()]
+    attrs.postinit(local_names)
+
+    builder_args = astroid.Tuple()
+    builder_args.postinit([cls.name, class_bases, attrs])
+
+    context.callcontext = astroid.context.CallContext(builder_args)
+    try:
+        inferred = next(func.infer_call_result(func, context), None)
+    except astroid.InferenceError:
+        return None
+    return inferred or None
+
+
 class TypeChecker(BaseChecker):
     """try to find bugs in the code using type inference
     """
@@ -296,6 +520,30 @@ accessed. Python regular expressions are accepted.'}
                          'to register other decorators that produce valid '
                          'context managers.'}
                ),
+               ('missing-member-hint-distance',
+                {'default': 1,
+                 'type': 'int',
+                 'metavar': '<member hint edit distance>',
+                 'help': 'The minimum edit distance a name should have in order '
+                         'to be considered a similar match for a missing member name.'
+                }
+               ),
+               ('missing-member-max-choices',
+                {'default': 1,
+                 'type': "int",
+                 'metavar': '<member hint max choices>',
+                 'help': 'The total number of similar names that should be taken in '
+                         'consideration when showing a hint for a missing member.'
+                }
+               ),
+               ('missing-member-hint',
+                {'default': True,
+                 'type': "yn",
+                 'metavar': '<missing member hint>',
+                 'help': 'Show a hint with possible names when a member name was not '
+                         'found. The aspect of finding the hint is based on edit distance.'
+                }
+               ),
               )
 
     def open(self):
@@ -307,9 +555,36 @@ accessed. Python regular expressions are accepted.'}
         if isinstance(self.config.generated_members, six.string_types):
             gen = shlex.shlex(self.config.generated_members)
             gen.whitespace += ','
-            gen.wordchars += '[]-+\.*?'
+            gen.wordchars += '[]-+\.*?()|'
             self.config.generated_members = tuple(tok.strip('"') for tok in gen)
 
+    @check_messages('invalid-metaclass')
+    def visit_classdef(self, node):
+
+        def _metaclass_name(metaclass):
+            if isinstance(metaclass, (astroid.ClassDef, astroid.FunctionDef)):
+                return metaclass.name
+            return metaclass.as_string()
+
+        metaclass = node.declared_metaclass()
+        if not metaclass:
+            return
+
+        if isinstance(metaclass, astroid.FunctionDef):
+            # Try to infer the result.
+            metaclass = _infer_from_metaclass_constructor(node, metaclass)
+            if not metaclass:
+                # Don't do anything if we cannot infer the result.
+                return
+
+        if isinstance(metaclass, astroid.ClassDef):
+            if _is_invalid_metaclass(metaclass):
+                self.add_message('invalid-metaclass', node=node,
+                                 args=(_metaclass_name(metaclass), ))
+        else:
+            self.add_message('invalid-metaclass', node=node,
+                             args=(_metaclass_name(metaclass), ))
+
     def visit_assignattr(self, node):
         if isinstance(node.assign_type(), astroid.AugAssign):
             self.visit_attribute(node)
@@ -387,9 +662,17 @@ accessed. Python regular expressions are accepted.'}
                     continue
                 done.add(actual)
                 confidence = INFERENCE if not inference_failure else INFERENCE_FAILURE
+
+                if self.config.missing_member_hint:
+                    hint = _missing_member_hint(owner, node.attrname,
+                                                self.config.missing_member_hint_distance,
+                                                self.config.missing_member_max_choices)
+                else:
+                    hint = ""
+
                 self.add_message('no-member', node=node,
                                  args=(owner.display_type(), name,
-                                       node.attrname),
+                                       node.attrname, hint),
                                  confidence=confidence)
 
     @check_messages('assignment-from-no-return', 'assignment-from-none')
@@ -409,7 +692,7 @@ accessed. Python regular expressions are accepted.'}
             return
         returns = list(function_node.nodes_of_class(astroid.Return,
                                                     skip_klass=astroid.FunctionDef))
-        if len(returns) == 0:
+        if not returns:
             self.add_message('assignment-from-no-return', node=node)
         else:
             for rnode in returns:
@@ -464,33 +747,6 @@ accessed. Python regular expressions are accepted.'}
                                      args=node.func.as_string())
                     break
 
-    @staticmethod
-    def _no_context_variadic(node):
-        """Verify if the given call node has variadic nodes without context
-
-        This is a workaround for handling cases of nested call functions
-        which don't have the specific call context at hand.
-        Variadic arguments (variable positional arguments and variable
-        keyword arguments) are inferred, inherently wrong, by astroid
-        as a Tuple, respectively a Dict with empty elements.
-        This can lead pylint to believe that a function call receives
-        too few arguments.
-        """
-        for arg in node.args:
-            if not isinstance(arg, astroid.Starred):
-                continue
-
-            inferred = safe_infer(arg.value)
-            if isinstance(inferred, astroid.Tuple):
-                length = len(inferred.elts)
-            elif isinstance(inferred, astroid.Dict):
-                length = len(inferred.items)
-            else:
-                return False
-            if not length and isinstance(inferred.statement(), astroid.FunctionDef):
-                return True
-        return False
-
     @check_messages(*(list(MSGS.keys())))
     def visit_call(self, node):
         """check that called functions/methods are inferred to callable objects,
@@ -502,13 +758,23 @@ accessed. Python regular expressions are accepted.'}
         call_site = astroid.arguments.CallSite.from_call(node)
         num_positional_args = len(call_site.positional_arguments)
         keyword_args = list(call_site.keyword_arguments.keys())
-        no_context_variadic = self._no_context_variadic(node)
+
+        # Determine if we don't have a context for our call and we use variadics.
+        if isinstance(node.scope(), astroid.FunctionDef):
+            has_no_context_positional_variadic = _no_context_variadic_positional(node)
+            has_no_context_keywords_variadic = _no_context_variadic_keywords(node)
+        else:
+            has_no_context_positional_variadic = has_no_context_keywords_variadic = False
 
         called = safe_infer(node.func)
         # only function, generator and object defining __call__ are allowed
-        if called is not None and not called.callable():
-            self.add_message('not-callable', node=node,
-                             args=node.func.as_string())
+        if called and not called.callable():
+            if isinstance(called, astroid.Instance) and not has_known_bases(called):
+                # Don't emit if we can't make sure this object is callable.
+                pass
+            else:
+                self.add_message('not-callable', node=node,
+                                 args=node.func.as_string())
 
         self._check_uninferable_callfunc(node)
 
@@ -636,13 +902,13 @@ accessed. Python regular expressions are accepted.'}
                 else:
                     display_name = repr(name)
                 # TODO(cpopa): this should be removed after PyCQA/astroid/issues/177
-                if not no_context_variadic:
+                if not has_no_context_positional_variadic:
                     self.add_message('no-value-for-parameter', node=node,
                                      args=(display_name, callable_name))
 
         for name in kwparams:
             defval, assigned = kwparams[name]
-            if defval is None and not assigned:
+            if defval is None and not assigned and not has_no_context_keywords_variadic:
                 self.add_message('missing-kwoa', node=node,
                                  args=(name, callable_name))
 
@@ -668,10 +934,9 @@ accessed. Python regular expressions are accepted.'}
         # Determine what method on the parent this index will use
         # The parent of this node will be a Subscript, and the parent of that
         # node determines if the Subscript is a get, set, or delete operation.
-        operation = node.parent.parent
-        if isinstance(operation, astroid.Assign):
+        if node.parent.ctx is astroid.Store:
             methodname = '__setitem__'
-        elif isinstance(operation, astroid.Delete):
+        elif node.parent.ctx is astroid.Del:
             methodname = '__delitem__'
         else:
             methodname = '__getitem__'
@@ -681,11 +946,13 @@ accessed. Python regular expressions are accepted.'}
         # type. This way we catch subclasses of sequence types but skip classes
         # that override __getitem__ and which may allow non-integer indices.
         try:
-            methods = parent_type.getattr(methodname)
+            methods = dunder_lookup.lookup(parent_type, methodname)
             if methods is astroid.YES:
                 return
             itemmethod = methods[0]
-        except (exceptions.NotFoundError, IndexError):
+        except (exceptions.NotFoundError,
+                exceptions.AttributeInferenceError,
+                IndexError):
             return
         if not isinstance(itemmethod, astroid.FunctionDef):
             return
@@ -810,9 +1077,8 @@ accessed. Python regular expressions are accepted.'}
                     self.add_message('not-context-manager',
                                      node=node, args=(infered.name, ))
 
-    # Disabled until we'll have a more capable astroid.
     @check_messages('invalid-unary-operand-type')
-    def _visit_unaryop(self, node):
+    def visit_unaryop(self, node):
         """Detect TypeErrors for unary operands."""
 
         for error in node.type_errors():
@@ -833,6 +1099,9 @@ accessed. Python regular expressions are accepted.'}
     def _check_binop_errors(self, node):
         for error in node.type_errors():
             # Let the error customize its output.
+            if any(isinstance(obj, astroid.ClassDef) and not has_known_bases(obj)
+                   for obj in (error.left_type, error.right_type)):
+                continue
             self.add_message('unsupported-binary-operation',
                              args=str(error), node=node)
 
@@ -853,32 +1122,42 @@ accessed. Python regular expressions are accepted.'}
     def visit_compare(self, node):
         if len(node.ops) != 1:
             return
-        operator, right = node.ops[0]
-        if operator in ['in', 'not in']:
+
+        op, right = node.ops[0]
+        if op in ['in', 'not in']:
             self._check_membership_test(right)
 
-    @check_messages('unsubscriptable-object')
+    @check_messages('unsubscriptable-object', 'unsupported-assignment-operation',
+                    'unsupported-delete-operation')
     def visit_subscript(self, node):
+        supported_protocol = None
         if isinstance(node.value, (astroid.ListComp, astroid.DictComp)):
             return
-        if isinstance(node.value, astroid.SetComp):
-            self.add_message('unsubscriptable-object',
-                             args=node.value.as_string(),
-                             node=node.value)
-            return
 
-        infered = safe_infer(node.value)
-        if infered is None or infered is astroid.YES:
+        if node.ctx == astroid.Load:
+            supported_protocol = supports_getitem
+            msg = 'unsubscriptable-object'
+        elif node.ctx == astroid.Store:
+            supported_protocol = supports_setitem
+            msg = 'unsupported-assignment-operation'
+        elif node.ctx == astroid.Del:
+            supported_protocol = supports_delitem
+            msg = 'unsupported-delete-operation'
+
+        if isinstance(node.value, astroid.SetComp):
+            self.add_message(msg, args=node.value.as_string(),
+                             node=node.value)
             return
 
         if is_inside_abstract_class(node):
             return
 
-        if not supports_subscript(infered):
-            self.add_message('unsubscriptable-object',
-                             args=node.value.as_string(),
-                             node=node.value)
+        inferred = safe_infer(node.value)
+        if inferred is None or inferred is astroid.YES:
+            return
 
+        if not supported_protocol(inferred):
+            self.add_message(msg, args=node.value.as_string(), node=node.value)
 
 
 class IterableChecker(BaseChecker):
@@ -898,11 +1177,11 @@ class IterableChecker(BaseChecker):
 
     msgs = {'E1133': ('Non-iterable value %s is used in an iterating context',
                       'not-an-iterable',
-                      'Used when a non-iterable value is used in place where'
+                      'Used when a non-iterable value is used in place where '
                       'iterable is expected'),
             'E1134': ('Non-mapping value %s is used in a mapping context',
                       'not-a-mapping',
-                      'Used when a non-mapping value is used in place where'
+                      'Used when a non-mapping value is used in place where '
                       'mapping is expected'),
            }
 
