@@ -1,5 +1,7 @@
-# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Copyright (c) 2006, 2010, 2012-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2013-2014 Google, Inc.
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
@@ -78,12 +80,6 @@ class BaseReporter(object):
             layout.children[0].children[0].data += ' (%s)' % layout.report_id
         self._display(layout)
 
-    def display_results(self, layout):
-        warnings.warn("display_results is deprecated, use display_reports instead. "
-                      "The former will be removed in Pylint 2.0.",
-                      DeprecationWarning)
-        self.display_reports(layout)
-
     def _display(self, layout):
         """display the layout"""
         raise NotImplementedError()
