@@ -1,24 +1,22 @@
-# Copyright (c) 2003-2014 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2012-2014 Google, Inc.
+# Copyright (c) 2014 Brett Cannon <brett@python.org>
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015 Aru Sahni <arusahni@gmail.com>
+# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
+# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
+
+# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+
 """some various utilities and helper classes, most of them used in the
 main pylint class
 """
 from __future__ import print_function
 
 import collections
+from inspect import cleandoc
 import os
 from os.path import dirname, basename, splitext, exists, isdir, join, normpath
 import re
@@ -31,18 +29,11 @@ import six
 from six.moves import zip  # pylint: disable=redefined-builtin
 
 from astroid import nodes, Module
-from astroid.modutils import modpath_from_file, get_module_files, \
-    file_from_modpath, load_module_from_file
+from astroid import modutils
 
 from pylint.interfaces import IRawChecker, ITokenChecker, UNDEFINED, implements
 from pylint.reporters.ureports.nodes import Section
-
-
-class UnknownMessage(Exception):
-    """raised when a unregistered message id is encountered"""
-
-class EmptyReport(Exception):
-    """raised when a report is empty and so should not be displayed"""
+from pylint.exceptions import InvalidMessageError, UnknownMessageError, EmptyReportError
 
 
 MSG_TYPES = {
@@ -64,20 +55,6 @@ MSG_TYPES_STATUS = {
     'F' : 1
     }
 
-DEPRECATED_ALIASES = {
-    # New name, deprecated name.
-    'repr': 'backquote',
-    'expr': 'discard',
-    'assignname': 'assname',
-    'assignattr': 'assattr',
-    'attribute': 'getattr',
-    'call': 'callfunc',
-    'importfrom': 'from',
-    'classdef': 'class',
-    'functiondef': 'function',
-    'generatorexp': 'genexpr',
-}
-
 _MSG_ORDER = 'EWRCIF'
 MSG_STATE_SCOPE_CONFIG = 0
 MSG_STATE_SCOPE_MODULE = 1
@@ -165,7 +142,7 @@ def build_message_def(checker, msgid, msg_tuple):
     if len(msg_tuple) > 3:
         (msg, symbol, descr, options) = msg_tuple
     elif len(msg_tuple) > 2:
-        (msg, symbol, descr) = msg_tuple[:3]
+        (msg, symbol, descr) = msg_tuple
     else:
         # messages should have a symbol, but for backward compatibility
         # they may not.
@@ -181,9 +158,11 @@ class MessageDefinition(object):
     def __init__(self, checker, msgid, msg, descr, symbol, scope,
                  minversion=None, maxversion=None, old_names=None):
         self.checker = checker
-        assert len(msgid) == 5, 'Invalid message id %s' % msgid
-        assert msgid[0] in MSG_TYPES, \
-               'Bad message type %s in %r' % (msgid[0], msgid)
+        if len(msgid) != 5:
+            raise InvalidMessageError('Invalid message id %r' % msgid)
+        if not msgid[0] in MSG_TYPES:
+            raise InvalidMessageError(
+                'Bad message type %s in %r' % (msgid[0], msgid))
         self.msgid = msgid
         self.msg = msg
         self.descr = descr
@@ -240,58 +219,79 @@ class MessagesHandlerMixIn(object):
         self.msg_status = 0
 
     def _checker_messages(self, checker):
-        for checker in self._checkers[checker.lower()]:
-            for msgid in checker.msgs:
+        for known_checker in self._checkers[checker.lower()]:
+            for msgid in known_checker.msgs:
                 yield msgid
 
     def disable(self, msgid, scope='package', line=None, ignore_unknown=False):
         """don't output message of the given id"""
+        self._set_msg_status(msgid, enable=False, scope=scope,
+                             line=line, ignore_unknown=ignore_unknown)
+
+    def enable(self, msgid, scope='package', line=None, ignore_unknown=False):
+        """reenable message of the given id"""
+        self._set_msg_status(msgid, enable=True, scope=scope,
+                             line=line, ignore_unknown=ignore_unknown)
+
+    def _set_msg_status(self, msgid, enable, scope='package', line=None, ignore_unknown=False):
         assert scope in ('package', 'module')
-        # handle disable=all by disabling all categories
+
         if msgid == 'all':
-            for msgid in MSG_TYPES:
-                self.disable(msgid, scope, line)
+            for _msgid in MSG_TYPES:
+                self._set_msg_status(_msgid, enable, scope, line, ignore_unknown)
+            if enable and not self._python3_porting_mode:
+                # Don't activate the python 3 porting checker if it wasn't activated explicitly.
+                self.disable('python3')
             return
+
         # msgid is a category?
         catid = category_id(msgid)
         if catid is not None:
             for _msgid in self.msgs_store._msgs_by_category.get(catid):
-                self.disable(_msgid, scope, line)
+                self._set_msg_status(_msgid, enable, scope, line)
             return
+
         # msgid is a checker name?
         if msgid.lower() in self._checkers:
             msgs_store = self.msgs_store
             for checker in self._checkers[msgid.lower()]:
                 for _msgid in checker.msgs:
                     if _msgid in msgs_store._alternative_names:
-                        self.disable(_msgid, scope, line)
+                        self._set_msg_status(_msgid, enable, scope, line)
             return
+
         # msgid is report id?
         if msgid.lower().startswith('rp'):
-            self.disable_report(msgid)
+            if enable:
+                self.enable_report(msgid)
+            else:
+                self.disable_report(msgid)
             return
 
         try:
             # msgid is a symbolic or numeric msgid.
             msg = self.msgs_store.check_message_id(msgid)
-        except UnknownMessage:
+        except UnknownMessageError:
             if ignore_unknown:
                 return
             raise
 
         if scope == 'module':
-            self.file_state.set_msg_status(msg, line, False)
-            if msg.symbol != 'locally-disabled':
+            self.file_state.set_msg_status(msg, line, enable)
+            if enable:
+                self.add_message('locally-enabled', line=line,
+                                 args=(msg.symbol, msg.msgid))
+            elif msg.symbol != 'locally-disabled':
                 self.add_message('locally-disabled', line=line,
                                  args=(msg.symbol, msg.msgid))
-
         else:
             msgs = self._msgs_state
-            msgs[msg.msgid] = False
+            msgs[msg.msgid] = enable
             # sync configuration object
-            self.config.disable = [self._message_symbol(mid)
-                                   for mid, val in six.iteritems(msgs)
-                                   if not val]
+            self.config.enable = [self._message_symbol(mid) for mid, val
+                                  in sorted(six.iteritems(msgs)) if val]
+            self.config.disable = [self._message_symbol(mid) for mid, val
+                                   in sorted(six.iteritems(msgs)) if not val]
 
     def _message_symbol(self, msgid):
         """Get the message symbol of the given message id
@@ -301,54 +301,9 @@ class MessagesHandlerMixIn(object):
         """
         try:
             return self.msgs_store.check_message_id(msgid).symbol
-        except UnknownMessage:
+        except UnknownMessageError:
             return msgid
 
-    def enable(self, msgid, scope='package', line=None, ignore_unknown=False):
-        """reenable message of the given id"""
-        assert scope in ('package', 'module')
-        if msgid == 'all':
-            for msgid_ in MSG_TYPES:
-                self.enable(msgid_, scope=scope, line=line)
-            if not self._python3_porting_mode:
-                # Don't activate the python 3 porting checker if it
-                # wasn't activated explicitly.
-                self.disable('python3')
-            return
-        catid = category_id(msgid)
-        # msgid is a category?
-        if catid is not None:
-            for msgid in self.msgs_store._msgs_by_category.get(catid):
-                self.enable(msgid, scope, line)
-            return
-        # msgid is a checker name?
-        if msgid.lower() in self._checkers:
-            for checker in self._checkers[msgid.lower()]:
-                for msgid_ in checker.msgs:
-                    self.enable(msgid_, scope, line)
-            return
-        # msgid is report id?
-        if msgid.lower().startswith('rp'):
-            self.enable_report(msgid)
-            return
-
-        try:
-            # msgid is a symbolic or numeric msgid.
-            msg = self.msgs_store.check_message_id(msgid)
-        except UnknownMessage:
-            if ignore_unknown:
-                return
-            raise
-
-        if scope == 'module':
-            self.file_state.set_msg_status(msg, line, True)
-            self.add_message('locally-enabled', line=line, args=(msg.symbol, msg.msgid))
-        else:
-            msgs = self._msgs_state
-            msgs[msg.msgid] = True
-            # sync configuration object
-            self.config.enable = [mid for mid, val in six.iteritems(msgs) if val]
-
     def get_message_state_scope(self, msgid, line=None, confidence=UNDEFINED):
         """Returns the scope at which a message was enabled/disabled."""
         if self.config.confidence and confidence.name not in self.config.confidence:
@@ -370,7 +325,7 @@ class MessagesHandlerMixIn(object):
                 return False
         try:
             msgid = self.msgs_store.check_message_id(msg_descr).msgid
-        except UnknownMessage:
+        except UnknownMessageError:
             # The linter checks for messages that are not registered
             # due to version mismatch, just treat them as message IDs
             # for now.
@@ -399,11 +354,18 @@ class MessagesHandlerMixIn(object):
         # does not apply to them.
         if msgid[0] not in _SCOPE_EXEMPT:
             if msg_info.scope == WarningScope.LINE:
-                assert node is None and line is not None, (
-                    'Message %s must only provide line, got line=%s, node=%s' % (msgid, line, node))
+                if line is None:
+                    raise InvalidMessageError(
+                        'Message %s must provide line, got None' % msgid)
+                if node is not None:
+                    raise InvalidMessageError(
+                        'Message %s must only provide line, '
+                        'got line=%s, node=%s' % (msgid, line, node))
             elif msg_info.scope == WarningScope.NODE:
                 # Node-based warnings may provide an override line.
-                assert node is not None, 'Message %s must provide Node, got None'
+                if node is None:
+                    raise InvalidMessageError(
+                        'Message %s must provide Node, got None' % msgid)
 
         if line is None and node is not None:
             line = node.fromlineno
@@ -443,13 +405,16 @@ class MessagesHandlerMixIn(object):
             Message(msgid, symbol,
                     (abspath, path, module, obj, line or 1, col_offset or 0), msg, confidence))
 
-    def print_full_documentation(self):
+    def print_full_documentation(self, stream=None):
         """output a full documentation in ReST format"""
-        print("Pylint global options and switches")
-        print("----------------------------------")
-        print("")
-        print("Pylint provides global options and switches.")
-        print("")
+        if not stream:
+            stream = sys.stdout
+
+        print("Pylint global options and switches", file=stream)
+        print("----------------------------------", file=stream)
+        print("", file=stream)
+        print("Pylint provides global options and switches.", file=stream)
+        print("", file=stream)
 
         by_checker = {}
         for checker in self.get_checkers():
@@ -460,63 +425,94 @@ class MessagesHandlerMixIn(object):
                             title = 'General options'
                         else:
                             title = '%s options' % section.capitalize()
-                        print(title)
-                        print('~' * len(title))
-                        _rest_format_section(sys.stdout, None, options)
-                        print("")
+                        print(title, file=stream)
+                        print('~' * len(title), file=stream)
+                        _rest_format_section(stream, None, options)
+                        print("", file=stream)
             else:
+                name = checker.name
                 try:
-                    by_checker[checker.name][0] += checker.options_and_values()
-                    by_checker[checker.name][1].update(checker.msgs)
-                    by_checker[checker.name][2] += checker.reports
+                    by_checker[name]['options'] += checker.options_and_values()
+                    by_checker[name]['msgs'].update(checker.msgs)
+                    by_checker[name]['reports'] += checker.reports
                 except KeyError:
-                    by_checker[checker.name] = [list(checker.options_and_values()),
-                                                dict(checker.msgs),
-                                                list(checker.reports)]
+                    by_checker[name] = {
+                        'options': list(checker.options_and_values()),
+                        'msgs':    dict(checker.msgs),
+                        'reports': list(checker.reports),
+                    }
 
-        print("Pylint checkers' options and switches")
-        print("-------------------------------------")
-        print("")
-        print("Pylint checkers can provide three set of features:")
-        print("")
-        print("* options that control their execution,")
-        print("* messages that they can raise,")
-        print("* reports that they can generate.")
-        print("")
-        print("Below is a list of all checkers and their features.")
-        print("")
+        print("Pylint checkers' options and switches", file=stream)
+        print("-------------------------------------", file=stream)
+        print("", file=stream)
+        print("Pylint checkers can provide three set of features:", file=stream)
+        print("", file=stream)
+        print("* options that control their execution,", file=stream)
+        print("* messages that they can raise,", file=stream)
+        print("* reports that they can generate.", file=stream)
+        print("", file=stream)
+        print("Below is a list of all checkers and their features.", file=stream)
+        print("", file=stream)
 
-        for checker, (options, msgs, reports) in six.iteritems(by_checker):
-            title = '%s checker' % (checker.replace("_", " ").title())
-            print(title)
-            print('~' * len(title))
-            print("")
-            print("Verbatim name of the checker is ``%s``." % checker)
-            print("")
-            if options:
-                title = 'Options'
-                print(title)
-                print('^' * len(title))
-                _rest_format_section(sys.stdout, None, options)
-                print("")
-            if msgs:
-                title = 'Messages'
-                print(title)
-                print('~' * len(title))
-                for msgid, msg in sorted(six.iteritems(msgs),
-                                         key=lambda kv: (_MSG_ORDER.index(kv[0][0]), kv[1])):
-                    msg = build_message_def(checker, msgid, msg)
-                    print(msg.format_help(checkerref=False))
-                print("")
-            if reports:
-                title = 'Reports'
-                print(title)
-                print('~' * len(title))
-                for report in reports:
-                    print(':%s: %s' % report[:2])
-                print("")
-            print("")
+        for checker, info in six.iteritems(by_checker):
+            self._print_checker_doc(checker, info, stream=stream)
 
+    @staticmethod
+    def _print_checker_doc(checker_name, info, stream=None):
+        """Helper method for print_full_documentation.
+
+        Also used by doc/exts/pylint_extensions.py.
+        """
+        if not stream:
+            stream = sys.stdout
+
+        doc = info.get('doc')
+        module = info.get('module')
+        msgs = info.get('msgs')
+        options = info.get('options')
+        reports = info.get('reports')
+
+        title = '%s checker' % (checker_name.replace("_", " ").title())
+
+        if module:
+            # Provide anchor to link against
+            print(".. _%s:\n" % module, file=stream)
+        print(title, file=stream)
+        print('~' * len(title), file=stream)
+        print("", file=stream)
+        if module:
+            print("This checker is provided by ``%s``." % module, file=stream)
+        print("Verbatim name of the checker is ``%s``." % checker_name, file=stream)
+        print("", file=stream)
+        if doc:
+            title = 'Documentation'
+            print(title, file=stream)
+            print('^' * len(title), file=stream)
+            print(cleandoc(doc), file=stream)
+            print("", file=stream)
+        if options:
+            title = 'Options'
+            print(title, file=stream)
+            print('^' * len(title), file=stream)
+            _rest_format_section(stream, None, options)
+            print("", file=stream)
+        if msgs:
+            title = 'Messages'
+            print(title, file=stream)
+            print('^' * len(title), file=stream)
+            for msgid, msg in sorted(six.iteritems(msgs),
+                                     key=lambda kv: (_MSG_ORDER.index(kv[0][0]), kv[1])):
+                msg = build_message_def(checker_name, msgid, msg)
+                print(msg.format_help(checkerref=False), file=stream)
+            print("", file=stream)
+        if reports:
+            title = 'Reports'
+            print(title, file=stream)
+            print('^' * len(title), file=stream)
+            for report in reports:
+                print(':%s: %s' % report[:2], file=stream)
+            print("", file=stream)
+        print("", file=stream)
 
 class FileState(object):
     """Hold internal state specific to the currently analyzed file"""
@@ -659,8 +655,8 @@ class MessagesStore(object):
         """
         msg = self.check_message_id(new_symbol)
         msg.old_names.append((old_id, old_symbol))
-        self._alternative_names[old_id] = msg
-        self._alternative_names[old_symbol] = msg
+        self._register_alternative_name(msg, old_id)
+        self._register_alternative_name(msg, old_symbol)
 
     def register_messages(self, checker):
         """register a dictionary of messages
@@ -672,29 +668,42 @@ class MessagesStore(object):
         are the checker id and the two last the message id in this checker
         """
         chkid = None
-        for msgid, msg_tuple in six.iteritems(checker.msgs):
+        for msgid, msg_tuple in sorted(six.iteritems(checker.msgs)):
             msg = build_message_def(checker, msgid, msg_tuple)
-            assert msg.symbol not in self._messages, \
-                    'Message symbol %r is already defined' % msg.symbol
             # avoid duplicate / malformed ids
-            assert msg.msgid not in self._alternative_names, \
-                   'Message id %r is already defined' % msgid
-            assert chkid is None or chkid == msg.msgid[1:3], \
-                   'Inconsistent checker part in message id %r' % msgid
+            if msg.symbol in self._messages or msg.symbol in self._alternative_names:
+                raise InvalidMessageError(
+                    'Message symbol %r is already defined' % msg.symbol)
+            if chkid is not None and chkid != msg.msgid[1:3]:
+                raise InvalidMessageError(
+                    "Inconsistent checker part in message id %r (expected 'x%sxx')"
+                    % (msgid, chkid))
             chkid = msg.msgid[1:3]
             self._messages[msg.symbol] = msg
-            self._alternative_names[msg.msgid] = msg
+            self._register_alternative_name(msg, msg.msgid)
             for old_id, old_symbol in msg.old_names:
-                self._alternative_names[old_id] = msg
-                self._alternative_names[old_symbol] = msg
+                self._register_alternative_name(msg, old_id)
+                self._register_alternative_name(msg, old_symbol)
             self._msgs_by_category[msg.msgid[0]].append(msg.msgid)
 
+    def _register_alternative_name(self, msg, name):
+        """helper for register_message()"""
+        if name in self._messages and self._messages[name] != msg:
+            raise InvalidMessageError(
+                'Message symbol %r is already defined' % name)
+        if name in self._alternative_names and self._alternative_names[name] != msg:
+            raise InvalidMessageError(
+                'Message %s %r is already defined' % (
+                    'id' if len(name) == 5 and name[0] in MSG_TYPES else 'alternate name',
+                    name))
+        self._alternative_names[name] = msg
+
     def check_message_id(self, msgid):
         """returns the Message object for this message.
 
         msgid may be either a numeric or symbolic id.
 
-        Raises UnknownMessage if the message id is not defined.
+        Raises UnknownMessageError if the message id is not defined.
         """
         if msgid[1:].isdigit():
             msgid = msgid.upper()
@@ -703,7 +712,7 @@ class MessagesStore(object):
                 return source[msgid]
             except KeyError:
                 pass
-        raise UnknownMessage('No such message id %s' % msgid)
+        raise UnknownMessageError('No such message id %s' % msgid)
 
     def get_msg_display_string(self, msgid):
         """Generates a user-consumable representation of a message.
@@ -718,7 +727,7 @@ class MessagesStore(object):
             try:
                 print(self.check_message_id(msgid).format_help(checkerref=True))
                 print("")
-            except UnknownMessage as ex:
+            except UnknownMessageError as ex:
                 print(ex)
                 print("")
                 continue
@@ -785,7 +794,7 @@ class ReportsHandlerMixIn(object):
                 report_sect = Section(r_title)
                 try:
                     r_cb(report_sect, stats, old_stats)
-                except EmptyReport:
+                except EmptyReportError:
                     continue
                 report_sect.report_id = reportid
                 sect.append(report_sect)
@@ -802,8 +811,29 @@ class ReportsHandlerMixIn(object):
             self.stats[key] = value
         return self.stats
 
+def _basename_in_blacklist_re(base_name, black_list_re):
+    """Determines if the basename is matched in a regex blacklist
 
-def expand_modules(files_or_modules, black_list):
+    :param str base_name: The basename of the file
+    :param list black_list_re: A collection of regex patterns to match against.
+        Successful matches are blacklisted.
+
+    :returns: `True` if the basename is blacklisted, `False` otherwise.
+    :rtype: bool
+    """
+    for file_pattern in black_list_re:
+        if file_pattern.match(base_name):
+            return True
+    return False
+
+def _modpath_from_file(filename, is_namespace):
+    def _is_package_cb(path, parts):
+        return modutils.check_modpath_has_init(path, parts) or is_namespace
+
+    return modutils.modpath_from_file_with_callback(filename, is_package_cb=_is_package_cb)
+
+
+def expand_modules(files_or_modules, black_list, black_list_re):
     """take a list of files/modules/packages and return the list of tuple
     (file, module name) which have to be actually checked
     """
@@ -813,7 +843,7 @@ def expand_modules(files_or_modules, black_list):
         if exists(something):
             # this is a file or a directory
             try:
-                modname = '.'.join(modpath_from_file(something))
+                modname = '.'.join(modutils.modpath_from_file(something))
             except ImportError:
                 modname = splitext(basename(something))[0]
             if isdir(something):
@@ -824,7 +854,7 @@ def expand_modules(files_or_modules, black_list):
             # suppose it's a module or package
             modname = something
             try:
-                filepath = file_from_modpath(modname.split('.'))
+                filepath = modutils.file_from_modpath(modname.split('.'))
                 if filepath is None:
                     continue
             except (ImportError, SyntaxError) as ex:
@@ -832,15 +862,37 @@ def expand_modules(files_or_modules, black_list):
                 # removed as soon as possible http://bugs.python.org/issue10588
                 errors.append({'key': 'fatal', 'mod': modname, 'ex': ex})
                 continue
+
         filepath = normpath(filepath)
-        result.append({'path': filepath, 'name': modname, 'isarg': True,
-                       'basepath': filepath, 'basename': modname})
-        if not (modname.endswith('.__init__') or modname == '__init__') \
-                and '__init__.py' in filepath:
-            for subfilepath in get_module_files(dirname(filepath), black_list):
+        modparts = (modname or something).split('.')
+
+        try:
+            spec = modutils.file_info_from_modpath(modparts, path=sys.path)
+        except ImportError:
+            # Might not be acceptable, don't crash.
+            is_namespace = False
+            is_directory = isdir(something)
+        else:
+            is_namespace = modutils.is_namespace(spec)
+            is_directory = modutils.is_directory(spec)
+
+        if not is_namespace:
+            result.append({'path': filepath, 'name': modname, 'isarg': True,
+                           'basepath': filepath, 'basename': modname})
+
+        has_init = (not (modname.endswith('.__init__') or modname == '__init__')
+                    and '__init__.py' in filepath)
+
+        if has_init or is_namespace or is_directory:
+            for subfilepath in modutils.get_module_files(dirname(filepath), black_list,
+                                                         list_all=is_namespace):
                 if filepath == subfilepath:
                     continue
-                submodname = '.'.join(modpath_from_file(subfilepath))
+                if _basename_in_blacklist_re(basename(subfilepath), black_list_re):
+                    continue
+
+                modpath = _modpath_from_file(subfilepath, is_namespace)
+                submodname = '.'.join(modpath)
                 result.append({'path': subfilepath, 'name': submodname,
                                'isarg': False,
                                'basepath': filepath, 'basename': modname})
@@ -904,29 +956,12 @@ class PyLintASTWalker(object):
         # Detect if the node is a new name for a deprecated alias.
         # In this case, favour the methods for the deprecated
         # alias if any,  in order to maintain backwards
-        # compatibility. If both of them are present,
-        # only the old ones will be called.
-        old_cid = DEPRECATED_ALIASES.get(cid)
+        # compatibility.
         visit_events = ()
         leave_events = ()
 
-        if old_cid:
-            visit_events = self.visit_events.get(old_cid)
-            leave_events = self.leave_events.get(old_cid)
-            if visit_events or leave_events:
-                msg = ("Implemented method {meth}_{old} instead of {meth}_{new}. "
-                       "This will be supported until Pylint 2.0.")
-                if visit_events:
-                    warnings.warn(msg.format(meth="visit", old=old_cid, new=cid),
-                                  PendingDeprecationWarning)
-                if leave_events:
-                    warnings.warn(msg.format(meth="leave", old=old_cid, new=cid),
-                                  PendingDeprecationWarning)
-
-        if not visit_events:
-            visit_events = self.visit_events.get(cid)
-        if not leave_events:
-            leave_events = self.leave_events.get(cid)
+        visit_events = self.visit_events.get(cid, ())
+        leave_events = self.leave_events.get(cid, ())
 
         if astroid.is_statement:
             self.nbstatements += 1
@@ -954,7 +989,7 @@ def register_plugins(linter, directory):
         if extension in PY_EXTS and base != '__init__' or (
                 not extension and isdir(join(directory, base))):
             try:
-                module = load_module_from_file(join(directory, filename))
+                module = modutils.load_module_from_file(join(directory, filename))
             except ValueError:
                 # empty module name (usually emacs auto-save files)
                 continue
@@ -988,11 +1023,10 @@ def get_global_option(checker, option, default=None):
     return default
 
 
-def deprecated_option(shortname=None, opt_type=None, help_msg=None):
+def deprecated_option(shortname=None, opt_type=None, help_msg=None, deprecation_msg=None):
     def _warn_deprecated(option, optname, *args): # pylint: disable=unused-argument
-        msg = ("Warning: option %s is obsolete and "
-               "it is slated for removal in Pylint 1.6.\n")
-        sys.stderr.write(msg % (optname,))
+        if deprecation_msg:
+            sys.stderr.write(deprecation_msg % (optname,))
 
     option = {
         'help': help_msg,
@@ -1086,14 +1120,14 @@ def _comment(string):
 def _format_option_value(optdict, value):
     """return the user input's value from a 'compiled' value"""
     if isinstance(value, (list, tuple)):
-        value = ','.join(value)
+        value = ','.join(_format_option_value(optdict, item) for item in value)
     elif isinstance(value, dict):
         value = ','.join('%s:%s' % (k, v) for k, v in value.items())
     elif hasattr(value, 'match'): # optdict.get('type') == 'regexp'
         # compiled regexp
         value = value.pattern
     elif optdict.get('type') == 'yn':
-        value = value and 'yes' or 'no'
+        value = 'yes' if value else 'no'
     elif isinstance(value, six.string_types) and value.isspace():
         value = "'%s'" % value
     return value
